import java.awt.Color;

import jrobots.landscape.TerrainType;
import jrobots.simulation.simulationObjects.JRobot2017;
import jrobots.utils.*;

/**
 * Keep it simple:<ul>
 * <li>if drving in wrong direction, do a U-turn.
 * <li>if crossing a border, turn 90 degrees
 * </ul>
 * @since 2017
 */
public class RacerSimple extends JRobot2017 {
    private static final long serialVersionUID = 1L;   
    
    private Angle targetDirection;    
    private TerrainType lastTerrain;
    private double timeOfLastDecision = Double.NEGATIVE_INFINITY;
    private static final double TIME_TO_KEEP_A_DECISION = 3; //seconds
    
    @Override
    protected void init() {
    	// on init: drive straight away
    	this.targetDirection = this.getOrientation();
    	this.lastTerrain = this.getCurrentTerrain();
    }
    
    @Override
    protected void actions() {   
    	// Is there another one first? Drive there!
    	if (!this.getPosOfBestBot().equals(this.getPosition())) {
    		this.targetDirection = 
    				this.getPosOfBestBot().sub(this.getPosition()).getAngle();
    		this.setBodyColor(Color.YELLOW);
    	}      	
    	// Decide new direction
    	else if (isTimeForNewDecision()) {
    		this.setBodyColor(Color.BLACK);
    		if (this.getIsMovingInWrongDirection()) {
    			// U-turn
    			this.targetDirection = this.targetDirection.getOpposite();
    		} else if (isTerrainWorseThanLastTime()) {
    			// Turn 90 degrees
    			this.targetDirection = this.targetDirection.getNextQuadrant();
    		} 		
    	} else {
    		this.setBodyColor(Color.WHITE);
    	}

		this.setAutopilot(this.targetDirection, 1d);
    }
    
    private boolean isTimeForNewDecision() {
    	if (this.getTime() > this.timeOfLastDecision + TIME_TO_KEEP_A_DECISION) {
    		this.timeOfLastDecision = this.getTime();
    		return true;
    	}    	
    	return false;
    }
    
    private boolean isTerrainWorseThanLastTime() {
    	TerrainType currentTerrain = this.getCurrentTerrain();
    	// ROAD < GRASS < SAND < OBSTACLE
    	boolean result = currentTerrain.ordinal() > this.lastTerrain.ordinal();
    	// memorize new terrain
		this.lastTerrain = this.getCurrentTerrain();
		return result;
    }
    
}
